"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = require("child_process");
const date_fns_1 = require("date-fns");
const fetch = require("node-fetch");
const includesOriginal = require("lodash.includes");
const includes = includesOriginal;
// Celebrate when a new release is being shipped
exports.checkForRelease = packageDiff => {
    if (packageDiff.version) {
        markdown(":tada:");
    }
};
// Initial stab at starting a new dependency information rule
// Just starting simple by showing `yarn why {dep}` for now
exports.checkForNewDependencies = (packageDiff) => __awaiter(this, void 0, void 0, function* () {
    const sentence = danger.utils.sentence;
    if (packageDiff.dependencies) {
        if (packageDiff.dependencies.added.length) {
            const newDependencies = packageDiff.dependencies.added;
            warn(`New dependencies added: ${sentence(newDependencies)}.`);
            for (const dep of newDependencies) {
                const output = child_process.execSync(`yarn why ${dep} --json`);
                let yarnDetails = "";
                if (output) {
                    // Comes as a series of little JSON messages
                    const usefulJSONContents = output.toString().split(`{"type":"activityEnd","data":{"id":0}}`).pop();
                    const asJSON = usefulJSONContents.split("}\n{").join("},{");
                    const whyJSON = JSON.parse(`[${asJSON}]`);
                    const messages = whyJSON.filter(msg => typeof msg.data === "string").map(m => m.data);
                    yarnDetails = `
<details>
  <summary><code>yarn why ${dep}</code> output</summary>
   <p><code><ul><li>${messages.join("</li><li>")}
   </li></ul></code></p>
</details>
`;
                }
                // Grab the NPM metadata
                let npmMetadata = "";
                const npmResponse = yield fetch(`https://registry.npmjs.org/${dep}`);
                if (npmResponse.ok) {
                    const tableDeets = [];
                    const npm = yield npmResponse.json();
                    if (npm.time && npm.time.created) {
                        const distance = date_fns_1.distanceInWords(new Date(npm.time.created), new Date());
                        tableDeets.push({ name: "Created", message: `${distance} ago` });
                    }
                    if (npm.time && npm.time.modified) {
                        const distance = date_fns_1.distanceInWords(new Date(npm.time.modified), new Date());
                        tableDeets.push({
                            name: "Last Updated",
                            message: `${distance} ago`,
                        });
                    }
                    if (npm.license) {
                        tableDeets.push({ name: "License", message: npm.license });
                    }
                    else {
                        tableDeets.push({
                            name: "License",
                            message: "<b>NO LICENSE FOUND</b>",
                        });
                    }
                    if (npm.maintainers) {
                        tableDeets.push({
                            name: "Maintainers",
                            message: npm.maintainers.length,
                        });
                    }
                    if (npm["dist-tags"] && npm["dist-tags"].latest) {
                        const currentTag = npm["dist-tags"].latest;
                        const tag = npm.versions[currentTag];
                        tableDeets.push({
                            name: "Releases",
                            message: String(Object.keys(npm.versions).length),
                        });
                        if (tag.dependencies) {
                            const deps = Object.keys(tag.dependencies);
                            const depLinks = deps.map(d => `<a href='http: //npmjs.com/package/${d}'>${d}</a>`);
                            tableDeets.push({
                                name: "Direct Dependencies",
                                message: sentence(depLinks),
                            });
                        }
                    }
                    if (npm.keywords && npm.keywords.length) {
                        tableDeets.push({
                            name: "Keywords",
                            message: sentence(npm.keywords),
                        });
                    }
                    let readme = "This README is too long to show.";
                    if (npm.readme && npm.readme.length < 10000) {
                        readme = `
<details>
<summary><code>README</code></summary>
${npm.readme}
</details>
`;
                    }
                    const homepage = npm.homepage ? npm.homepage : `http: //npmjs.com/package/${dep}`;
                    npmMetadata = `
<h2><a href="${homepage}">${dep}</a></h2>
Author: ${npm.author && npm.author.name ? npm.author.name : "Unknown"}
Description: ${npm.description}
Repo: ${homepage}
<table>
  <thead><tr><th></th><th width="100%"></th></tr></thead>
  ${tableDeets.map(deet => `<tr><td>${deet.name}</td><td>${deet.message}</td></tr>`).join("")}
</table>
${readme}
`;
                }
                else {
                    const errorMessage = yield npmResponse.text();
                    npmMetadata = `Error getting NPM details: ${errorMessage}`;
                }
                markdown(`${npmMetadata} ${yarnDetails}`);
            }
        }
    }
});
// Ensure a lockfile change if deps/devDeps changes, in case
// someone has only used `npm install` instead of `yarn.
exports.checkForLockfileDiff = packageDiff => {
    if (packageDiff.dependencies || packageDiff.devDependencies) {
        const lockfileChanged = includes(danger.git.modified_files, "yarn.lock");
        if (!lockfileChanged) {
            const message = "Changes were made to package.json, but not to yarn.lock.";
            const idea = "Perhaps you need to run `yarn install`?";
            warn(`${message}<br/><i>${idea}</i>`);
        }
    }
};
// Don't ship @types dependencies to consumers of Danger
exports.checkForTypesInDeps = (packageDiff) => {
    const sentence = danger.utils.sentence;
    if (packageDiff.dependencies) {
        const typesDeps = packageDiff.dependencies.added.filter(dep => dep.startsWith("@types"));
        if (typesDeps.length) {
            const message = `@types dependencies were added to package.json, as a dependency for others.`;
            const idea = `You need to move ${sentence(typesDeps)} into "devDependencies"?`;
            fail(`${message}<br/><i>${idea}</i>`);
        }
    }
};
/**
 * Provides dependency information on dependency changes in a PR
 */
function yarn(pathToPackageJSON) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = pathToPackageJSON ? pathToPackageJSON : "package.json";
        const packageDiff = yield danger.git.JSONDiffForFile(path);
        exports.checkForRelease(packageDiff);
        yield exports.checkForNewDependencies(packageDiff);
        exports.checkForLockfileDiff(packageDiff);
        exports.checkForTypesInDeps(packageDiff);
    });
}
exports.default = yarn;
